from functools import wraps
from django.http import Http404
from django.http import HttpResponse

from .utils import function_from_string

def _return_blank_or_raise_404(is_ajax):
    """
    Return empty string '' in case of ajax requests and raise 404 in case
    of non-ajax requests.
    """
    if is_ajax:
        return HttpResponse('')
    else:
        raise Http404


def quick(config, mysterious=None, only_authenticated=None):
    """
    Decides whether this user is allowed to access this view or not.

    :param config - Decides if the setting is on globally.
    :callable_name - The function which will return the list of users which are
                     eligible for proceeding further after this action.
    """

    def decorator(func):

        @wraps(func)
        def _quick(request, *args, **kwargs):
            # Check if the request is ajax.
            is_ajax = request.is_ajax()

            # Check if the config is available globally and return '' or raise
            # 404 as per the nature of the request.
            if not config:
                return _return_blank_or_raise_404(is_ajax)
            callable_name = None
            _only_authenticated = None

            if mysterious is not None:
                if type(mysterious) == bool:
                    _only_authenticated = mysterious
                else:
                    callable_name = mysterious
            elif only_authenticated is not None:
                _only_authenticated = only_authenticated

            user = request.user
            if callable_name is None:
                if (_only_authenticated is not None and
                        _only_authenticated and
                        not user.is_authenticated()):
                    return _return_blank_or_raise_404(is_ajax)
                else:
                    return func(request, *args, **kwargs)
            else:
                if not user.is_authenticated():
                    return _return_blank_or_raise_404(is_ajax)
                else:
                    _callable = function_from_string(callable_name)
                    if user.id in _callable():
                        return func(request, *args, **kwargs)
                    return _return_blank_or_raise_404(is_ajax)
            return _return_blank_or_raise_404(is_ajax)
        return _quick
    return decorator



from django import template
from django.template.base import TemplateSyntaxError

from ..utils import function_from_string

register = template.Library()

"""
class TemplateIfExperimentParser(IfParser):
    error_class = TemplateSyntaxError
    def __init__(self, parser, *args, **kwargs):
        self.template_parser = parser
        super(TemplateIfExperimentParser, self).__init__(*args, **kwargs)

    def create_var(self, value):
        pass
"""

class IfExperimentNode(template.Node):
    def __init__(self, nodelists):
        self.nodelists = nodelists

    def _set_user(self, context):
        request = template.Variable('request').resolve(context)
        if request.user.is_authenticated():
            self._user_id = request.user.id
            self._user_authenticated = True
        else:
            self._user_authenticated = False

    def prepare_experiment_params(self, bits, context):
        config = template.Variable(bits[0]).resolve(context)
        callable_name = None
        only_authenticated = None
        lbits = bits.__len__()
        if lbits > 1:
            mysterious = bits[1]
            if type(mysterious) == bool:
                only_authenticated = mysterious
            else:
                callable_name = mysterious
        elif lbits > 2:
            callable_name = bits[1]
            only_authenticated = bits[2]
        return config, callable_name, only_authenticated


    def matches_experiment(self, bits, context):
        if not bits:
            return True


        tup = self.prepare_experiment_params(bits, context)
        config = tup[0]

        if not config:
            return False

        callable_name = tup[1]
        only_authenticated = tup[2]

        if callable_name is None:
            if (only_authenticated is not None and
                    only_authenticated and
                    not self._user_authenticated):
                return False
            else:
                return True
        else:
            if not self._user_authenticated:
                return False
            else:
                _callable = function_from_string(callable_name)
                return self._user_id in _callable()
        return match

    def render(self, context):
        self._set_user(context)
        for nodelist, bits in self.nodelists:
            if self.matches_experiment(bits, context):
                return nodelist.render(context)
        return ''

@register.tag
def ifexperiment(parser, token):
    bits = token.split_contents()[1:]
    nodelist = parser.parse(('elifexperiment', 'else', 'endifexperiment'))
    nodelists = [(nodelist, bits)]
    token = parser.next_token()

    # {% elif ... %} (repeatable)
    while token.contents.startswith('elifexperiment'):
        bits = token.split_contents()[1:]
        nodelist = parser.parse(('elifexperiment', 'else', 'endifexperiment'))
        nodelists.append((nodelist, bits))
        token = parser.next_token()

    # {% else %} (optional)
    if token.contents == 'else':
        nodelist = parser.parse(('endifexperiment',))
        nodelists.append((nodelist, []))
        token = parser.next_token()

    # {% endif %}
    if token.contents != 'endifexperiment':
        raise TemplateSyntaxError('Malformed template tag at line {0}: "{1}"'.format(token.lineno, token.contents))

    return IfExperimentNode(nodelists)



from importlib import import_module
from django.conf import settings

def experiment_access(request):
    ctx = {}
    experiments = settings.__dict__.keys()
    for experiment in experiments:
        if not experiment.startswith('F_'):
            continue
        ctx[experiment] = getattr(settings, experiment, False)
    return ctx



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

def reqfile(filepath):
    """Turns a text file into a list (one element per line)"""
    result = []
    import re
    url_re = re.compile(".+:.+#egg=(.+)")
    with open(filepath, "r") as f:
        for line in f:
            line = line.strip()
            if not line or line.startswith("#"):
                continue
            mo = url_re.match(line)
            if mo is not None:
                line = mo.group(1)
            result.append(line)
    return result


setup(
    name="django-ab",
    version="0.1.1",
    description="AB Testing Framework for Django",
    long_description=open('README.md').read(),
    author='Sayan Chowdhury, Virendra Jain',
    author_email='sayanchowdhury@fedoraproject.org, virendra2334@gmail.com',
    url="https://github.com/djangothon/ab",
    license="AGPLv3",
    classifiers=[
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Programming Language :: Python :: 2",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=reqfile("requirements.txt"),
)



from importlib import import_module


def function_from_string(func_string):
    """
    Returns a function object from the function string.
    :param the string which needs to be resolved.
    """

    func = None
    func_string_splitted = func_string.split('.')
    module_name = '.'.join(func_string_splitted[:-1])
    function_name = func_string_splitted[-1]
    module = import_module(module_name)
    if module and function_name:
        func = getattr(module, function_name)
    return func
